@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2017-2018 Brazil
-  Copyright(C) 2018 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2018-2020 Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -179,6 +179,20 @@ grn_table_module_set_proc(grn_ctx *ctx,
   CRITICAL_SECTION_LEAVE(module->lock);
 }
 
+static void
+grn_table_module_set_options_without_lock(grn_ctx *ctx,
+                                          grn_table_module *module,
+                                          void *options,
+                                          grn_option_revision revision,
+                                          grn_close_func close_func)
+{
+  grn_table_module_fin_options(ctx, module);
+
+  module->options = options;
+  module->options_revision = revision;
+  module->options_close_func = close_func;
+}
+
 void
 grn_table_module_set_options(grn_ctx *ctx,
                              grn_table_module *module,
@@ -187,11 +201,11 @@ grn_table_module_set_options(grn_ctx *ctx,
                              grn_close_func close_func)
 {
   CRITICAL_SECTION_ENTER(module->lock);
-  grn_table_module_fin_options(ctx, module);
-
-  module->options = options;
-  module->options_revision = revision;
-  module->options_close_func = close_func;
+  grn_table_module_set_options_without_lock(ctx,
+                                            module,
+                                            options,
+                                            revision,
+                                            close_func);
   CRITICAL_SECTION_LEAVE(module->lock);
 }
 
@@ -359,28 +373,32 @@ grn_table_cache_module_options(grn_ctx *ctx,
   }
 
   GRN_TEXT_INIT(&raw_options, GRN_OBJ_VECTOR);
+  CRITICAL_SECTION_ENTER(module->lock);
   revision = grn_obj_get_option_values(ctx,
                                        table,
                                        data->module_name,
                                        -1,
                                        module->options_revision,
                                        &raw_options);
-  if ((revision == GRN_OPTION_REVISION_UNCHANGED) ||
-      (revision == GRN_OPTION_REVISION_NONE && module->options)) {
-    goto exit;
+  bool need_update = true;
+  if (revision == GRN_OPTION_REVISION_UNCHANGED) {
+    need_update = false;
   }
-
-  options = data->open_options_func(ctx,
-                                    module->proc,
-                                    &raw_options,
-                                    data->user_data);
-  grn_table_module_set_options(ctx,
-                               module,
-                               options,
-                               revision,
-                               data->close_options_func);
-
-exit :
+  if (revision == GRN_OPTION_REVISION_NONE && module->options) {
+    need_update = false;
+  }
+  if (need_update) {
+    options = data->open_options_func(ctx,
+                                      module->proc,
+                                      &raw_options,
+                                      data->user_data);
+    grn_table_module_set_options_without_lock(ctx,
+                                              module,
+                                              options,
+                                              revision,
+                                              data->close_options_func);
+  }
+  CRITICAL_SECTION_LEAVE(module->lock);
   GRN_OBJ_FIN(ctx, &raw_options);
 
   GRN_API_RETURN(module->options);
@@ -919,7 +937,7 @@ grn_table_get_score(grn_ctx *ctx, grn_obj *table, grn_id id)
   grn_rset_recinfo *ri =
     (grn_rset_recinfo *)grn_obj_get_value_(ctx, table, id, &value_size);
   double score = ri->score;
-  grn_obj *parent;
+  grn_obj *parent = NULL;
   grn_id parent_record_id;
   if (grn_table_get_key(ctx,
                         table,
@@ -932,9 +950,8 @@ grn_table_get_score(grn_ctx *ctx, grn_obj *table, grn_id id)
     return score;
   }
 
-  for (parent = grn_ctx_at(ctx, table->header.domain);
-       parent && (parent->header.flags & GRN_OBJ_WITH_SUBREC);
-       parent = grn_ctx_at(ctx, parent->header.domain)) {
+  parent = grn_ctx_at(ctx, table->header.domain);
+  while (parent && (parent->header.flags & GRN_OBJ_WITH_SUBREC)) {
     uint32_t parent_value_size;
     grn_rset_recinfo *parent_ri =
       (grn_rset_recinfo *)grn_obj_get_value_(ctx,
@@ -955,6 +972,16 @@ grn_table_get_score(grn_ctx *ctx, grn_obj *table, grn_id id)
     if (parent_record_id == GRN_ID_NIL) {
       break;
     }
+
+    grn_id next_parent_domain = parent->header.domain;
+    if (grn_enable_reference_count) {
+      grn_obj_unlink(ctx, parent);
+    }
+    parent = grn_ctx_at(ctx, next_parent_domain);
+  }
+  if (parent && grn_enable_reference_count) {
+    grn_obj_unlink(ctx, parent);
+    parent = NULL;
   }
 
   return score;
